/* RPO-01-2xTimerInt.c  17.02.2015
*
* pic18F87J11 (PICDEM PIC18 Explorer)
*
* LED toggle control on background
*
* Timer0, interrupt high priority
* Timer1, interrupt low priority
*/

//Includes
#include <p18F87J11.h>

//Function prototypes
void ledControl(void);
void Interrupt_low (void);
void Interrupt_high (void);
void Timer0_int (void);
void Timer1_int (void);

//new type definitions
typedef int Tboolean;

//Defines
enum{FALSE, TRUE};
#define LED0_ON_COUNT 8
#define LED0_OFF_COUNT 5

//Global variables
int x=0;
Tboolean timeXmsTick = FALSE;
char led_toggle0=0, led_toggle1=0;
char led0On, led0Toggle, led0ToggleOld;
int led0OnCntr=0, led0OffCntr=0;

int count0=1000, count1=15;

//#pragma code
void main (void) ///////////////////////////////////////////////////////////////
{
  // If TRISx bit is cleared, then the bit is an output bit. 

	TRISD = 0b11110000;			// Configure RB0,1,2,3 as outputs

	// Reset LEDs 
	PORTDbits.RD0 = 0;
	PORTDbits.RD1 = 0;
	PORTDbits.RD2 = 0;
	PORTDbits.RD3 = 0;
	
	//Led0 control vars
	led0On = TRUE;
	led0Toggle = FALSE;
	led0ToggleOld = led0Toggle;
	led0OnCntr=0;
	led0OffCntr=0;

	// Timer 0
	T0CON &= 0b11011111;			// TMR0 internal clock
	INTCON2bits.TMR0IP = 1;		// TMR0 high priority interrupt group
	INTCONbits.TMR0IE=1;			// Enable TMR0 interrupt

	// Timer 1
	T1CONbits.RD16 = 1;
	T1CONbits.TMR1ON = 1; 		// Enable Timer1
	TMR1H = 0;
	TMR1L = 0;
	IPR1bits.TMR1IP = 0;			// Low priority interrupt
	PIE1bits.TMR1IE = 1; 			// Enable Timer1 interrupt

	RCONbits.IPEN = 1;				// Enable priority feature
	INTCONbits.GIEH = 1;			// Enable high priority interrupts
	INTCONbits.GIEL = 1; 			// Enable low prioritz interrupts

	//Timer0_int();

  while(TRUE){
		if(timeXmsTick){
			timeXmsTick = FALSE;
			ledControl();
		}
	}//while()END
    
}// main() END /////////////////////////////////////////////////////////////////


void ledControl(void)//////////////////////////////////////////////////////////
{
if(led0Toggle == TRUE && led0ToggleOld != led0Toggle){
		//Rising edge
		led0ToggleOld = led0Toggle;
		led0On = FALSE;
		led0OffCntr = LED0_OFF_COUNT;
		led0OnCntr = 0;
	}
	
	if(led0Toggle == FALSE && led0ToggleOld != led0Toggle){
		//Falling edge
		led0ToggleOld = led0Toggle;
		led0On = TRUE;
		led0OffCntr = 0;
		led0OnCntr = 0;
	}

	if(led0Toggle == TRUE){

		if(led0OffCntr > 0){
			led0OffCntr--;
			if(led0OffCntr == 0){
				led0On = TRUE;
				led0OnCntr = LED0_ON_COUNT;
			}
		}
		
		if(led0OnCntr > 0){
			led0OnCntr--;
			if(led0OnCntr == 0){
				led0On = FALSE;
				led0OffCntr = LED0_OFF_COUNT;
			}
		}
	
	}//led toggle part end

	//Send led0On to the port
}// ledControl() END /////////////////////////////////////////////////////////////////


#pragma interrupt Timer0_int
void Timer0_int (void) /////////////////////////////////////////////////////////
{
	INTCONbits.TMR0IF=0;
	//if(--count0) return;
	//count0 = 10;

	timeXmsTick = TRUE;

	x++; //dbg only
}// Timer0_int() END ///////////////////////////////////////////////////////////


#pragma interruptlow Timer1_int
void Timer1_int (void) /////////////////////////////////////////////////////////
{
	PIR1bits.TMR1IF = 0;	// Cleare interrupt flag
	x++;									// 40MHz/4 = 10MHz -> 16bit overflow = 6.5526ms
	if(--count1) return;
	count1 = 15;
	if(led_toggle1)
		{
		led_toggle1=0;
		PORTDbits.RD2 = 1;
		PORTDbits.RD3 = 0;
		}
	else
		{
		led_toggle1=1;
		PORTDbits.RD2 = 0;
		PORTDbits.RD3 = 1;
		}
}// Timer1_int() END ///////////////////////////////////////////////////////////


#pragma code int_high=0x000008
void Interrupt_high (void) /////////////////////////////////////////////////////
{
	_asm GOTO Timer0_int _endasm
}// Interrupt_high() END ///////////////////////////////////////////////////////


#pragma code int_low=0x000018
void Interrupt_low (void) //////////////////////////////////////////////////////
{
	_asm GOTO Timer1_int _endasm
}// Interrupt_low() END ////////////////////////////////////////////////////////


